@@ -21,9 +21,7 @@ BEGIN {
 use Test::Unit::Lite;
 
 use Exception::Base
-    max_arg_nums => 0, max_arg_len => 200, verbosity => 3,
-    '+ignore_package' => [ qr/^Test::Unit::/, 'main' ],
-    'Exception::Base::Died';
+    max_arg_nums => 0, max_arg_len => 200, verbosity => 4;
 use Exception::Warning '%SIG' => 'die';
 
 all_tests;
